/*
 * Parse slash-delimited book data.
 */
%%

%class lex

%standalone

%unicode

%line
%column
%char

LineTerminator = \r|\n|\r\n
Slash = "/"
Field = [^/\r\n]+
Title = \"{Field}\"

%{

   StringBuffer line;

   private void initialize_buffer(String s) {
       int adjusted_line_no = yyline + 1;
       line = new StringBuffer("[" + adjusted_line_no + "] " + s);
   }

   private void add_identifier_to_buffer(String s) {
       line.append(s);
   }

   private void add_delimiter_to_buffer(String s) {
       line.append(s);
   }

   private void display_line() {
       System.out.println(line.toString());
   }

   private void wrap_up() {
       System.out.println("Lines Processed: " + yyline);
       System.out.println("Total Number of Characters: " + yychar);
   }
%}

%%

{Title}            { initialize_buffer(yytext()); }

{Field}            { add_identifier_to_buffer(yytext()); }

{Slash}            { add_delimiter_to_buffer("/"); }

{LineTerminator}   { display_line(); }

<<EOF>>            { wrap_up(); return 0; }
