/*
 * Parse slash-delimited book data.
 */
%%

%class lex

%standalone

%unicode

%line
%column
%char

LineTerminator = \r|\n|\r\n
Slash = "/"
Field = [^/\r\n]+
Title = \"[^/\r\n]+\"

%{

   boolean at_start = true;
   
   private void print_xml_declaration() {
       if (at_start) {
           System.out.print("<?xml version='1.0'?>");
       }
   }
   
   private void print_root_start_tag(String s) {
       if (at_start) {
           System.out.print("<" + s + ">");
           at_start = false;
       }
   }

   private void print_root_end_tag(String s) {
       System.out.print("</" + s + ">");
   }

   private void print_start_tag(String s) {
       System.out.print("<" + s + ">");
   }

   private void print_element_data(String s) {
       System.out.print(s);
   }

   private void print_end_tag(String s) {
       System.out.print("</" + s + ">");
   }

   private void print_new_line() {
       System.out.println();
   }
%}

%%

{Title}            { print_xml_declaration();
                     print_root_start_tag("BookCatalogue");
                     print_start_tag("Book");
                     print_start_tag("Title"); 
                     print_element_data(yytext());
                     print_end_tag("Title");
                   }

{Field}            { print_start_tag("Field"); 
                     print_element_data(yytext());
                     print_end_tag("Field");
                   }

{Slash}            { print_new_line(); }

{LineTerminator}   { print_end_tag("Book");
                     print_new_line(); 
                   }

<<EOF>>            { print_root_end_tag("BookCatalogue");
                     return 0; 
                   }
