/*
 * Parse slash-delimited book data.
 */
%%

%class lex

%standalone

%unicode

%line
%column
%char

%state TITLE, AUTHOR, DATE, ISBN, PUBLISHER

LineTerminator = \r|\n|\r\n
Slash = "/"
Field = [^/\r\n]+
Title = \"[^/\r\n]+\"
Title_Missing_Left_Quote = [^/\r\n]+\"
Title_Missing_Right_Quote = \"[^/\r\n]+
Char  = .

%{
   private void print_xml_declaration() {
       System.out.print("<?xml version='1.0'?>");
   }

   private void print_start_tag(String s) {
       System.out.print("<" + s + ">");
   }

   private void print_element_data(String s) {
       System.out.print(s);
   }

   private void print_end_tag(String s) {
       System.out.print("</" + s + ">");
   }

   private void print_new_line() {
       System.out.println();
   }

   private void warning(String s) {
       System.err.println(s);
   }
%}

%%

<YYINITIAL> {

    {Char}              { print_xml_declaration();
                          print_start_tag("BookCatalogue");
                          yypushback(1);
                          yybegin(TITLE);
                        }
}

<TITLE> {

    {Title}             { print_start_tag("Book");
                          print_start_tag("Title"); 
                          print_element_data(yytext());
                          print_end_tag("Title");
                        }

    {Title_Missing_Left_Quote}  { warning("Missing left quote in Title");
                                  print_start_tag("Book");
                                  print_start_tag("Title"); 
                                  print_element_data("\"" + yytext());
                                  print_end_tag("Title");
                                }

    {Title_Missing_Right_Quote}  { warning("Missing right quote in Title");
                                  print_start_tag("Book");
                                   print_start_tag("Title"); 
                                   print_element_data(yytext() + "\"");
                                   print_end_tag("Title");
                                 }

    {Slash}             { yybegin(AUTHOR); }

    <<EOF>>             { print_end_tag("BookCatalogue");
                          return 0; 
                        }

}

<AUTHOR> {

    {Field}             { print_start_tag("Author"); 
                          print_element_data(yytext());
                          print_end_tag("Author");
                        }

    {Slash}             { yybegin(DATE); }

}

<DATE> {

    {Field}             { print_start_tag("Date"); 
                          print_element_data(yytext());
                          print_end_tag("Date");
                        }

    {Slash}             { yybegin(ISBN); }

}

<ISBN> {

    {Field}             { print_start_tag("ISBN"); 
                          print_element_data(yytext());
                          print_end_tag("ISBN");
                        }

    {Slash}             { yybegin(PUBLISHER); }

}

<PUBLISHER> {

    {Field}             { print_start_tag("Publisher"); 
                          print_element_data(yytext());
                          print_end_tag("Publisher");
                          print_end_tag("Book");
                        }

    {LineTerminator}    { yybegin(TITLE); }
}
