/*
 * Parse slash-delimited book data.
 */
%%

%class lex

%standalone

%unicode

%line
%column
%char

%state TITLE, AUTHOR, DATE, ISBN, PUBLISHER, MAGAZINE, MAGAZINEDATE

LineTerminator = \r|\n|\r\n
Slash = "/"
Field = [^/\r\n]+
Char  = .

%{
   boolean isBook = false;

   private void print_xml_declaration() {
       System.out.print("<?xml version='1.0'?>");
   }

   private void print_start_tag(String s) {
       System.out.print("<" + s + ">");
   }

   private void print_element_data(String s) {
       System.out.print(s);
   }

   private void print_end_tag(String s) {
       System.out.print("</" + s + ">");
   }
%}

%%

<YYINITIAL> {

    {Char}              { print_xml_declaration();
                          print_start_tag("Publications");
                          yypushback(1);
                          yybegin(TITLE);
                        }
}

<TITLE> {

    \"{Field}\"/{Slash}{Field}{Slash}{Field}{Slash}{Field}{Slash}{Field}(\r|\n|\r\n) 
                        { print_start_tag("Book");
                          print_start_tag("Title"); 
                          print_element_data(yytext());
                          print_end_tag("Title");
                          isBook = true;
                        }

    \"{Field}\"/{Slash}{Field}{Slash}{Field}(\r|\n|\r\n)
                        { print_start_tag("Magazine");
                          print_start_tag("Title"); 
                          print_element_data(yytext());
                          print_end_tag("Title");
                          isBook = false;
                        }

    {Slash}             { if (isBook) {
                              yybegin(AUTHOR); 
                          }
                          else {
                              yybegin(MAGAZINE);
                          }
                        }

    <<EOF>>             { print_end_tag("Publications");
                          return 0; 
                        }

}

<AUTHOR> {

    {Field}             { print_start_tag("Author"); 
                          print_element_data(yytext());
                          print_end_tag("Author");
                        }

    {Slash}             { yybegin(DATE); }

}

<DATE> {

    {Field}             { print_start_tag("Date"); 
                          print_element_data(yytext());
                          print_end_tag("Date");
                        }

    {Slash}             { yybegin(ISBN); }

}

<ISBN> {

    {Field}             { print_start_tag("ISBN"); 
                          print_element_data(yytext());
                          print_end_tag("ISBN");
                        }

    {Slash}             { yybegin(PUBLISHER); }

}

<PUBLISHER> {

    {Field}             { print_start_tag("Publisher"); 
                          print_element_data(yytext());
                          print_end_tag("Publisher");
                          print_end_tag("Book");
                        }

    {LineTerminator}    { yybegin(TITLE); }
}

<MAGAZINE> {

    {Field}             { print_start_tag("Name"); 
                          print_element_data(yytext());
                          print_end_tag("Name");
                        }

    {Slash}             { yybegin(MAGAZINEDATE); }

}

<MAGAZINEDATE> {

    {Field}             { print_start_tag("Date"); 
                          print_element_data(yytext());
                          print_end_tag("Date");
                          print_end_tag("Magazine");
                        }

    {LineTerminator}    { yybegin(TITLE); }
}
