import java_cup.runtime.*;

%%

%class lex
%cup

%standalone

%unicode

LineTerminator = \r|\n|\r\n
XML_Declaration = "<?xml version="[\"']"1.0"[\"'][^\r\n]*"?>"{LineTerminator}
DOCTYPE_Declaration = "<!DOCTYPE "[^\r\n<>]+">"{LineTerminator}
Name = [^</>]+
StartTag = "<"{Name}">"
EndTag = "</"{Name}">"
PCDATA = [a-zA-Z0-9;& ,._\n\r\t-]+
Whitespace = [\n\r\t ]+

%%

{XML_Declaration}       { return new Symbol(sym.XML_DECLARATION, yytext()); }

{DOCTYPE_Declaration}   { return new Symbol(sym.DOCTYPE_DECLARATION, yytext()); }

{StartTag}              { String token = yytext();
                          if (token.equals("<BookCatalogue>")) return new Symbol(sym.BookCatalogue_ST, yytext());
                          else if (token.equals("<Book>")) return new Symbol(sym.Book_ST, yytext());
                          else if (token.equals("<Title>")) return new Symbol(sym.Title_ST, yytext());
                          else if (token.equals("<Author>")) return new Symbol(sym.Author_ST, yytext());
                          else if (token.equals("<Date>")) return new Symbol(sym.Date_ST, yytext());
                          else if (token.equals("<ISBN>")) return new Symbol(sym.ISBN_ST, yytext());
                          else if (token.equals("<Publisher>")) return new Symbol(sym.Publisher_ST, yytext());
                          else return new Symbol(sym.START_TAG, yytext()); 
                        }

{EndTag}                { String token = yytext();
                          if (token.equals("</BookCatalogue>")) return new Symbol(sym.BookCatalogue_ET, yytext());
                          else if (token.equals("</Book>")) return new Symbol(sym.Book_ET, yytext());
                          else if (token.equals("</Title>")) return new Symbol(sym.Title_ET, yytext());
                          else if (token.equals("</Author>")) return new Symbol(sym.Author_ET, yytext());
                          else if (token.equals("</Date>")) return new Symbol(sym.Date_ET, yytext());
                          else if (token.equals("</ISBN>")) return new Symbol(sym.ISBN_ET, yytext());
                          else if (token.equals("</Publisher>")) return new Symbol(sym.Publisher_ET, yytext());
                          else return new Symbol(sym.START_TAG, yytext()); 
                        }

{Whitespace}            { ; }  /* Drop whitespace */

{PCDATA}                { return new Symbol(sym.PCDATA, yytext()); }

<<EOF>>                 { return new Symbol(sym.EOF); }
