/*
 * Parse newline-delimited juicer data.
 */
%%

%class lex

%standalone

%unicode

%line

juicer = "Juicer:"
name = "name = "[^\r\n]*
type = "type = "[^\r\n]*
image = "image = "[^\r\n]*
description = "description = "[^\r\n]*
warranty = "warranty = "[^\r\n]*
weight = "weight(lbs) = "[^\r\n]*
cost = "cost(USD) = "[^\r\n]*
retailer = "retailer = "[^\r\n]*
error = [^\r\n]+

%{

private String getData(String line) {
    int idx = line.indexOf(" = ");
    return line.substring(idx+3);
}

%}

%%

{juicer}                  { System.out.println("<juicer>"); }

{name}                    { System.out.println("<name>" + getData(yytext()) + "</name>"); }

{type}                    { System.out.println("<type>" + getData(yytext()) + "</type>");}

{image}                   { System.out.println("<image>" + getData(yytext()) + "</image>"); }

{description}             { System.out.println("<description>" + getData(yytext()) + "</description>"); }

{warranty}                { System.out.println("<warranty>" + getData(yytext()) + "</warranty>"); }

{weight}                  { System.out.println("<weight units='lbs'>" + getData(yytext()) + "</weight>"); }

{cost}                    { System.out.println("<cost currency='USD'>" + getData(yytext()) + "</cost>"); }

{retailer}                { System.out.println("<retailer>" + getData(yytext()) + "</retailer>"); }

[-]+                      { System.out.println("</juicer>"); }

\r|\n|\r\n                { ; /* Do nothing */}

{error}                   { System.err.println("Error in input text on line: " + yyline); }

<<EOF>>                   { return 0; }
