/*
 * Parse newline-delimited juicer data.
 */
%%

%class lex

%standalone

%unicode

%line

char = [^\r\n]
juicer = "Juicer:"
name = "name = "{char}*
type = "type = "{char}*
image = "image = "{char}*
description = "description = "{char}*
warranty = "warranty = "{char}*
weight = "weight(lbs) = "{char}*
cost = "cost(USD) = "{char}*
retailer = "retailer = "{char}*
error = {char}+

%{

boolean atStart = true;

private String getData(String line) {
    int idx = line.indexOf(" = ");
    return line.substring(idx+3);
}

%}

%%

{juicer}                  { if (atStart) {
                                System.out.println("<juicers>");
                                atStart = false;
                            }
                            System.out.println("<juicer>"); 
                          }

{name}                    { System.out.println("<name>" + getData(yytext()) + "</name>"); }

{type}                    { System.out.println("<type>" + getData(yytext()) + "</type>");}

{image}                   { System.out.println("<image>" + getData(yytext()) + "</image>"); }

{description}             { System.out.println("<description>" + getData(yytext()) + "</description>"); }

{warranty}                { System.out.println("<warranty>" + getData(yytext()) + "</warranty>"); }

{weight}                  { System.out.println("<weight units='lbs'>" + getData(yytext()) + "</weight>"); }

{cost}                    { System.out.println("<cost currency='USD'>" + getData(yytext()) + "</cost>"); }

{retailer}                { System.out.println("<retailer>" + getData(yytext()) + "</retailer>"); }

[-]+                      { System.out.println("</juicer>"); }

\r|\n|\r\n                { ; /* Do nothing */}

{error}                   { System.err.println("Error in input text on line: " + yyline); }

<<EOF>>                   { System.out.println("</juicers>");
                            return 0; 
                          }
