import java_cup.runtime.*;

%%

%class lex
%cup

%standalone

%unicode

LineTerminator = \r|\n|\r\n
XML_Declaration = "<?xml version="[\"']"1.0"[\"'][^\r\n]*"?>"{LineTerminator}
DOCTYPE_Declaration = "<!DOCTYPE "[^\r\n<>]+">"{LineTerminator}
Name = [^</>]+
StartTag = "<"{Name}">"
EndTag = "</"{Name}">"
PCDATA = [a-zA-Z0-9;& :,._()\"%\n\r\t-]+
Whitespace = [\n\r\t ]+

%%

{XML_Declaration}       { return new Symbol(sym.XML_DECLARATION, yytext()); }

{DOCTYPE_Declaration}   { return new Symbol(sym.DOCTYPE_DECLARATION, yytext()); }

{StartTag}              { String token = yytext();
                          if (token.equals("<juicers>")) return new Symbol(sym.JUICERS_ST);
                          else if (token.equals("<juicer>")) return new Symbol(sym.JUICER_ST);
                          else if (token.equals("<name>")) return new Symbol(sym.NAME_ST);
                          else if (token.equals("<type>")) return new Symbol(sym.TYPE_ST);
                          else if (token.equals("<image>")) return new Symbol(sym.IMAGE_ST);
                          else if (token.equals("<description>")) return new Symbol(sym.DESCRIPTION_ST);
                          else if (token.equals("<warranty>")) return new Symbol(sym.WARRANTY_ST);
                          else if (token.equals("<weight>")) return new Symbol(sym.WEIGHT_ST);
                          else if (token.equals("<cost>")) return new Symbol(sym.COST_ST);
                          else if (token.equals("<retailer>")) return new Symbol(sym.RETAILER_ST);
                        }

{EndTag}                { String token = yytext();
                          if (token.equals("</juicers>")) return new Symbol(sym.JUICERS_ET);
                          else if (token.equals("</juicer>")) return new Symbol(sym.JUICER_ET);
                          else if (token.equals("</name>")) return new Symbol(sym.NAME_ET);
                          else if (token.equals("</type>")) return new Symbol(sym.TYPE_ET);
                          else if (token.equals("</image>")) return new Symbol(sym.IMAGE_ET);
                          else if (token.equals("</description>")) return new Symbol(sym.DESCRIPTION_ET);
                          else if (token.equals("</warranty>")) return new Symbol(sym.WARRANTY_ET);
                          else if (token.equals("</weight>")) return new Symbol(sym.WEIGHT_ET);
                          else if (token.equals("</cost>")) return new Symbol(sym.COST_ET);
                          else if (token.equals("</retailer>")) return new Symbol(sym.RETAILER_ET);
                        }

{Whitespace}            { ; }  /* Drop whitespace */

{PCDATA}                { return new Symbol(sym.PCDATA, yytext()); }

<<EOF>>                 { return new Symbol(sym.EOF); }
